import requests
from bs4 import BeautifulSoup
import pandas as pd
import time
import json

# Fonction pour récupérer toutes les URLs d'une page
def get_urls(url):
    try:
        response = requests.get(url)
        soup = BeautifulSoup(response.content, 'html.parser')
        urls = [a['href'] for a in soup.find_all('a', href=True) if a['href'].startswith('https://www.jeremy-vincent.com')]
        return urls
    except Exception as e:
        print(f"Error fetching URLs from {url}: {e}")
        return []

# Fonction pour extraire le contenu d'une page
def extract_content(url):
    try:
        response = requests.get(url)
        soup = BeautifulSoup(response.content, 'html.parser')
        
        title = soup.title.string if soup.title else 'No title'
        meta_desc = soup.find('meta', attrs={'name': 'description'})
        meta_desc = meta_desc['content'] if meta_desc else 'No meta description'
        
        content = []
        headings = soup.find_all(['h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'p'])
        
        current_heading = None
        
        for tag in headings:
            if tag.name in ['h1', 'h2', 'h3', 'h4', 'h5', 'h6']:
                current_heading = tag.get_text()
                content.append({'heading': current_heading, 'text': ''})
            elif tag.name == 'p' and current_heading is not None:
                if content[-1]['text']:
                    content[-1]['text'] += ' ' + tag.get_text()
                else:
                    content[-1]['text'] = tag.get_text()
        
        content_str = '\n'.join([f"{item['heading']}:\n{item['text']}" for item in content])

        return {
            'URL': url,
            'Title': title,
            'Meta Description': meta_desc,
            'Content': content_str
        }
    except Exception as e:
        print(f"Error extracting content from {url}: {e}")
        return None

# Point de départ
start_url = 'https://www.jeremy-vincent.com'

# Structures de données pour le suivi
visited_urls = set()
urls_to_visit = set([start_url])

# Chargement des données sauvegardées précédemment
try:
    with open('jeremyvincent_visited_urls.json', 'r') as f:
        visited_urls = set(json.load(f))
    data = pd.read_csv('jeremyvincent_content.csv').to_dict('records')
except FileNotFoundError:
    data = []

# Compteur pour le nombre de pages crawled
counter = len(visited_urls)

try:
    while urls_to_visit:
        current_url = urls_to_visit.pop()
        if current_url in visited_urls:
            continue
        
        visited_urls.add(current_url)
        print(f"Crawling: {current_url}")
        urls_to_visit.update(get_urls(current_url))
        
        page_data = extract_content(current_url)
        if page_data:
            data.append(page_data)
        
        counter += 1
        
        # Sauvegarder les données et l'état toutes les 10 pages crawled
        if counter % 10 == 0:
            df = pd.DataFrame(data)
            df.to_csv('jeremyvincent_content.csv', index=False, encoding='utf-8')
            with open('jeremyvincent_visited_urls.json', 'w') as f:
                json.dump(list(visited_urls), f)
            with open('jeremyvincent_urls_to_visit.json', 'w') as f:
                json.dump(list(urls_to_visit), f)
    
        time.sleep(1)  # Attendre une seconde entre les requêtes pour ne pas surcharger le serveur

except KeyboardInterrupt:
    print("Crawling interrompu par l'utilisateur. Sauvegarde des données en cours...")
    df = pd.DataFrame(data)
    df.to_csv('jeremyvincent_content.csv', index=False, encoding='utf-8')
    with open('jeremyvincent_visited_urls.json', 'w') as f:
        json.dump(list(visited_urls), f)
    with open('jeremyvincent_urls_to_visit.json', 'w') as f:
        json.dump(list(urls_to_visit), f)

# Sauvegarder les données restantes
df = pd.DataFrame(data)
df.to_csv('jeremyvincent_content.csv', index=False, encoding='utf-8')
with open('jeremyvincent_visited_urls.json', 'w') as f:
    json.dump(list(visited_urls), f)
with open('jeremyvincent_urls_to_visit.json', 'w') as f:
    json.dump(list(urls_to_visit), f)

print("Extraction terminée et sauvegardée dans jeremyvincent_content.csv")
